package com.bocloud.mcs.collector.cloudtower;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.model.Butler;
import com.bocloud.cmp.provider.CloudTowerProvider;
import com.bocloud.common.encrypt.DESEncrptor;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.common.utils.Common;
import com.bocloud.ims.entity.CloudVendor;
import com.bocloud.ims.enums.CloudProvider;
import com.bocloud.mcs.collector.BocloudCollector;
import com.bocloud.service.core.CurrentService;
import com.smartx.tower.api.MetricsApi;
import lombok.extern.slf4j.Slf4j;
import org.apache.http.ssl.SSLContextBuilder;
import org.openstack4j.core.transport.Config;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Async;
import org.springframework.scheduling.annotation.Scheduled;

import java.security.KeyManagementException;
import java.security.KeyStoreException;
import java.security.NoSuchAlgorithmException;
import java.util.*;

@Slf4j
public abstract class CloudTowerCollector extends BocloudCollector {

    protected static Config config;
    static {
        // 全部信任 不做身份鉴定
        try {
            SSLContextBuilder contextBuilder = SSLContextBuilder.create().loadTrustMaterial(null,
                    (x509Certificates, authType) -> true);
            config = Config.newConfig().withSSLVerificationDisabled().withSSLContext(contextBuilder.build())
                    .withReadTimeout(20 * 1000 * 1000).withConnectionTimeout(20 * 1000 * 1000);
        } catch (NoSuchAlgorithmException e) {
            log.error("NoSuchAlgorithmException", e);
        } catch (KeyStoreException e) {
            log.error("KeyStoreException", e);
        } catch (KeyManagementException e) {
            log.error("KeyManagementException", e);
        }
    }

    /**
     * 缓存过期时间，30分钟
     *
     * @return
     */
    @Override
    public int getExpire() {
        return 1800000;
    }

    Map<String, Object> cache = new HashMap<>();

    @Autowired
    private CurrentService currentService;


    @Scheduled(initialDelay = 5000L, fixedRate = 300000L)
    @Async
    @Override
    public void scheduledCollect() {
        if (!currentService.isLeader()) {
            log.info(this.getClass().getName() + "当前节点是否是主节点:" + currentService.isLeader());
            return;
        }
        Date now = new Date();
        log.info(this.getClass().getName() + " metrics start...");
        BocloudResult bocloudResult = cmpCloudVendorService.list(CloudProvider.CLOUDTOWER);
        if (bocloudResult.isFailed()) {
            log.error("get cloud vendor list failed , error:{}", bocloudResult.getMessage());
            return;
        }
        JSONObject data = (JSONObject) bocloudResult.getData();
        List<CloudVendor> listCloudVendor = data.getJSONArray(Common.ROWS).toJavaList(CloudVendor.class);

        List<MetricFamilySamples> mfs = new ArrayList<>();
        for (CloudVendor cloudVendor : listCloudVendor) {
            try {
                String address = JSONObject.parseObject(cloudVendor.getAuthentication()).getString("address");
                String username = JSONObject.parseObject(cloudVendor.getAuthentication()).getString("account");
                String passwordJson = JSONObject.parseObject(cloudVendor.getAuthentication()).getString("password");
                String password = new DESEncrptor().decrypt(passwordJson.trim(), "BocloudCMP");
                Butler butler = new Butler();
                butler.setUrl(address);
                butler.setUsername(username);
                butler.setPassword(password);
                CloudTowerProvider cloudTowerProvider =  new CloudTowerProvider(butler);
                boolean accessable = cloudTowerProvider.accessable();
                if (accessable) {
                    MetricsApi metricsApi = new MetricsApi(cloudTowerProvider.getApiClient());
                    collectMfs(mfs, metricsApi, cloudVendor);
                }
            } catch (Exception e) {
                log.error("同步监控指标报错", e);
            }
        }
        cache.put(TIME, new Date());
        cache.put(DATA, mfs);
        log.info(this.getClass().getName() + " metrics end! use " + (System.currentTimeMillis() - now.getTime()) / 1000 + "s");
    }

    protected abstract void collectMfs(List<MetricFamilySamples> mfs, MetricsApi metricsApi, CloudVendor cloudVendor) throws Exception;

    @Override
    public Map<String, Object> getCache() {
        return this.cache;
    }
}
