package com.bocloud.mcs.collector.cloudtower;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.ims.entity.CloudVendor;
import com.bocloud.mcs.common.Constants;
import com.google.common.util.concurrent.ThreadFactoryBuilder;
import com.smartx.tower.api.MetricsApi;
import com.smartx.tower.model.GetHostMetricInput;
import com.smartx.tower.model.HostWhereInput;
import com.smartx.tower.model.Metric;
import com.smartx.tower.model.WithTaskMetric;
import io.prometheus.client.GaugeMetricFamily;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;

import java.util.*;
import java.util.concurrent.*;

@Slf4j
@Component
public class CloudTowerHostCollector extends CloudTowerCollector {
    ThreadFactory nameThreadFactory = new ThreadFactoryBuilder().setNameFormat("cloudTower-host-%d").build();
    private ExecutorService executorService = new ThreadPoolExecutor(Runtime.getRuntime().availableProcessors() + 1, Runtime.getRuntime().availableProcessors() + 1,
            0L, TimeUnit.MILLISECONDS,
            new LinkedBlockingQueue<>(), nameThreadFactory);
    @Override
    protected void collectMfs(List<MetricFamilySamples> mfs, MetricsApi metricsApi, CloudVendor cloudVendor) {
        Date now = new Date();
        List<String> labels = Arrays.asList("vendorId", "id", "name", "instanceId");
        GaugeMetricFamily collectTimeUse = new GaugeMetricFamily(Constants.CLOUDTOWERHOSTPREFIX + "collect_time_use", "help",
                Arrays.asList("type"));
        try {
            BocloudResult bocloudResult = cmpCloudVendorService.hostList(cloudVendor);
            JSONObject data = (JSONObject) bocloudResult.getData();
            if (data == null) {
                log.info("查询cloudTower宿主机为null");
                return;
            }
            JSONArray objects = JSONObject.parseArray(data.get("rows").toString());
            List<String> instanceIds = new ArrayList<>();
            for (Object object : objects) {
                JSONObject server = JSONObject.parseObject(object.toString());
                instanceIds.add(server.getString("uuid"));
            }
            String [] metrics = {
                    "host_cpu_overall_usage_percent",
                    "elf_host_vcpus_provisioned",
                    "host_hp_memory_usage_percent",
                    "elf_host_memory_provisioned_bytes",
                    // TODO
                    "zbs_chunk_data_capacity_bytes",
                    "zbs_chunk_used_data_space_bytes",
                    "zbs_chunk_read_iops",
                    "zbs_chunk_write_iops",
                    "zbs_chunk_avg_read_latency_ns",
                    "zbs_chunk_avg_write_latency_ns"
            };
            log.info("get {} cloudTowerHOST ", objects.size());
            List<Future<List<MetricFamilySamples>>> futures = new ArrayList<>();
            for (Object object : objects) {
                Future<List<MetricFamilySamples>> future = executorService.submit(() -> {
                    List<MetricFamilySamples> mfsOne = new ArrayList<>();
                    // cpu使用率
                    GaugeMetricFamily cpusUsed = new GaugeMetricFamily(Constants.CLOUDTOWERHOSTPREFIX + "host_cpu_overall_usage_percent", "help",
                            labels);
                    // cpu已使用量
                    GaugeMetricFamily cpusUsedTotal = new GaugeMetricFamily(Constants.CLOUDTOWERHOSTPREFIX + "elf_host_vcpus_provisioned", "help",
                            labels);
                    // 内存使用率
                    GaugeMetricFamily memUsed = new GaugeMetricFamily(Constants.CLOUDTOWERHOSTPREFIX + "host_memory_usage_percent", "help",
                            labels);
                    // 内存已使用量
                    GaugeMetricFamily memUsedTotal = new GaugeMetricFamily(Constants.CLOUDTOWERHOSTPREFIX + "elf_host_memory_provisioned_bytes", "help",
                            labels);

                    // 存储使用率
                    GaugeMetricFamily storageUsed = new GaugeMetricFamily(Constants.CLOUDTOWERHOSTPREFIX + "host_storage_usage_percent", "help",
                            labels);
                    // 存储已使用量
                    GaugeMetricFamily storageUsedTotal = new GaugeMetricFamily(Constants.CLOUDTOWERHOSTPREFIX + "zbs_chunk_used_data_space_bytes", "help",
                            labels);

                    GaugeMetricFamily storeIOPSRead = new GaugeMetricFamily(Constants.CLOUDTOWERHOSTPREFIX + "zbs_chunk_read_iops", "help",
                            labels);
                    // 存储吞吐量（读）
                    GaugeMetricFamily storeIngestionRead = new GaugeMetricFamily(Constants.CLOUDTOWERHOSTPREFIX + "zbs_chunk_write_iops", "help",
                            labels);
                    // 主机I/O平均读延时
                    GaugeMetricFamily storeAwaitRead = new GaugeMetricFamily(Constants.CLOUDTOWERHOSTPREFIX + "zbs_chunk_avg_read_latency_ns", "help",
                            labels);
                    // 主机I/O平均写延时
                    GaugeMetricFamily storeIOPSWrite = new GaugeMetricFamily(Constants.CLOUDTOWERHOSTPREFIX + "zbs_chunk_avg_write_latency_ns", "help",
                            labels);
                    JSONObject server = JSONObject.parseObject(object.toString());
                    if (server.getString("uuid") == null) {
                        log.info("当前主机信息不完善{}" ,server.getString("name"));
                    } else {
                        GetHostMetricInput getHostMetricInput = new GetHostMetricInput();
                        getHostMetricInput.setMetrics(Arrays.asList(metrics));
                        getHostMetricInput.setRange("2h");
                        GetHostMetricInput monitors = getHostMetricInput.hosts(new HostWhereInput().id(server.getString("uuid")));
                        List<WithTaskMetric> hostMetrics = metricsApi.getHostMetrics(monitors);
                        double storeTotal = 0;
                        for (WithTaskMetric monitor : hostMetrics) {
                            if (monitor.getData() == null){
                                continue;
                            }
                            Metric metric = monitor.getData();
                            if (metric.getSampleStreams().get(0).getPoints() == null) {
                                continue;
                            }
                            // cpu使用率
                            if ("host_cpu_overall_usage_percent".equals(metric.getSampleStreams().get(0).getLabels().getMetricName())) {
                                // 取时间最大值
                                double value = getV(metric);
                                cpusUsed.addMetric(Arrays.asList(String.valueOf(cloudVendor.getId()), server.get("id").toString(),
                                        server.get("name").toString(), server.get("uuid").toString()), value);
                            }
                            // cpu使用量
                            if ("elf_host_vcpus_provisioned".equals(metric.getSampleStreams().get(0).getLabels().getMetricName())) {
                                // 取时间最大值
                                double value = getV(metric);
                                cpusUsedTotal.addMetric(Arrays.asList(String.valueOf(cloudVendor.getId()), server.get("id").toString(),
                                        server.get("name").toString(), server.get("uuid").toString()), value);
                            }
                            // 内存使用率
                            if ("host_hp_memory_usage_percent".equals(metric.getSampleStreams().get(0).getLabels().getMetricName())) {
                                // 取时间最大值
                                double value = getV(metric);
                                memUsed.addMetric(Arrays.asList(String.valueOf(cloudVendor.getId()), server.get("id").toString(),
                                        server.get("name").toString(), server.get("uuid").toString()), value);
                            }
                            // 存储使用量
                            if ("zbs_chunk_used_data_space_bytes".equals(metric.getSampleStreams().get(0).getLabels().getMetricName())) {
                                // 取时间最大值
                                double storeUsedTotalValue = getV(metric);
                                // 存储使用率
                                storageUsed.addMetric(Arrays.asList(String.valueOf(cloudVendor.getId()), server.get("id").toString(),
                                        server.get("name").toString(), server.get("uuid").toString()), (storeUsedTotalValue/storeTotal)*100);
                                // 存储使用量
                                storageUsedTotal.addMetric(Arrays.asList(String.valueOf(cloudVendor.getId()), server.get("id").toString(),
                                        server.get("name").toString(), server.get("uuid").toString()), storeUsedTotalValue);
                            }
                            if ("elf_host_memory_provisioned_bytes".equals(metric.getSampleStreams().get(0).getLabels().getMetricName())) {
                                // 取时间最大值
                                double value = getV(metric);
                                memUsedTotal.addMetric(Arrays.asList(String.valueOf(cloudVendor.getId()), server.get("id").toString(),
                                        server.get("name").toString(), server.get("uuid").toString()), value);
                            }
                            if ("zbs_chunk_read_iops".equals(metric.getSampleStreams().get(0).getLabels().getMetricName())) {
                                // 取时间最大值
                                double value = getV(metric);
                                storeIOPSRead.addMetric(Arrays.asList(String.valueOf(cloudVendor.getId()), server.get("id").toString(),
                                        server.get("name").toString(), server.get("uuid").toString()), value);
                            }
                            if ("zbs_chunk_write_iops".equals(metric.getSampleStreams().get(0).getLabels().getMetricName())) {
                                // 取时间最大值
                                double value = getV(metric);
                                storeIngestionRead.addMetric(Arrays.asList(String.valueOf(cloudVendor.getId()), server.get("id").toString(),
                                        server.get("name").toString(), server.get("uuid").toString()), value);
                            }
                            if ("zbs_chunk_avg_read_latency_ns".equals(metric.getSampleStreams().get(0).getLabels().getMetricName())) {
                                // 取时间最大值
                                double value = getV(metric);
                                storeAwaitRead.addMetric(Arrays.asList(String.valueOf(cloudVendor.getId()), server.get("id").toString(),
                                        server.get("name").toString(), server.get("uuid").toString()), value);
                            }
                            if ("zbs_chunk_avg_write_latency_ns".equals(metric.getSampleStreams().get(0).getLabels().getMetricName())) {
                                // 取时间最大值
                                double value = getV(metric);
                                storeIOPSWrite.addMetric(Arrays.asList(String.valueOf(cloudVendor.getId()), server.get("id").toString(),
                                        server.get("name").toString(), server.get("uuid").toString()), value);
                            }
                        }
                        mfsOne.add(cpusUsed);
                        mfsOne.add(cpusUsedTotal);
                        mfsOne.add(memUsed);
                        mfsOne.add(memUsedTotal);
                        mfsOne.add(storageUsed);
                        mfsOne.add(storageUsedTotal);
                        mfsOne.add(storeIOPSRead);
                        mfsOne.add(storeIngestionRead);
                        mfsOne.add(storeAwaitRead);
                        mfsOne.add(storeIOPSWrite);
                    }
                    return mfsOne;
                });
                futures.add(future);
            }
            Optional.ofNullable(futures).orElse(new ArrayList<>()).forEach(stringFuture -> {
                try {
                    mfs.addAll(stringFuture.get());
                } catch (Exception e) {
                    log.error("collect cloudTower host partition metrics error!" + e.getMessage(), e);
                }
            });
            collectTimeUse.addMetric(Arrays.asList("cloudTower_host"), (System.currentTimeMillis() - now.getTime()));
            mfs.add(collectTimeUse);
        } catch (Exception e) {
            log.error(e.getMessage());
        }
        log.info("collect cloudTower host partition metrics end! use " + (System.currentTimeMillis() - now.getTime()) / 1000 + "s");
    }

    private static Double getV(Metric metric) {
        return metric.getSampleStreams().get(metric.getSampleStreams().size() - 1)
                .getPoints()
                .get(metric.getSampleStreams().get(0).getPoints().size() - 1)
                .getV();
    }
}
