package com.bocloud.mcs.collector.cloudtower;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.ims.entity.CloudVendor;
import com.bocloud.mcs.common.Constants;
import com.google.common.util.concurrent.ThreadFactoryBuilder;
import com.smartx.tower.api.MetricsApi;
import com.smartx.tower.model.*;
import io.prometheus.client.GaugeMetricFamily;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;

import java.util.*;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.*;

@Slf4j
@Component
public class CloudTowerHostNetCollector extends CloudTowerCollector {
    ThreadFactory nameThreadFactory = new ThreadFactoryBuilder().setNameFormat("cloudTower-hostNet-%d").build();
    private ExecutorService executorService = new ThreadPoolExecutor(Runtime.getRuntime().availableProcessors() + 1, Runtime.getRuntime().availableProcessors() + 1,
            0L, TimeUnit.MILLISECONDS,
            new LinkedBlockingQueue<>(), nameThreadFactory);
    @Override
    protected void collectMfs(List<MetricFamilySamples> mfs, MetricsApi metricsApi, CloudVendor cloudVendor) {
        Date now = new Date();
        List<String> labels = Arrays.asList("vendorId", "id", "name", "instanceId");
        // 网口数据接收速度
        GaugeMetricFamily networkReceive = new GaugeMetricFamily(Constants.CLOUDTOWERHOSTPREFIX + "host_network_receive_speed_bitps", "help",labels);
        // 网口数据发送速度
        GaugeMetricFamily networkTransmit = new GaugeMetricFamily(Constants.CLOUDTOWERHOSTPREFIX + "host_network_transmit_speed_bitps", "help",labels);

        GaugeMetricFamily collectTimeUse = new GaugeMetricFamily(Constants.CLOUDTOWERHOSTPREFIX + "collect_time_use", "help",
                Arrays.asList("type"));
        try {
            BocloudResult bocloudResult = cmpCloudVendorService.hostList(cloudVendor);
            JSONObject data = (JSONObject) bocloudResult.getData();
            if (data == null) {
                log.info("查询cloudTower宿主机为null");
                return;
            }
            JSONArray objects = JSONObject.parseArray(data.get("rows").toString());
            List<String> instanceIds = new ArrayList<>();
            for (Object object : objects) {
                JSONObject server = JSONObject.parseObject(object.toString());
                instanceIds.add(server.getString("uuid"));
            }
            String [] netMetrics = {
                    // 网口数据接收速度
                    "host_network_receive_speed_bitps",
                    // 网口数据发送速度
                    "host_network_transmit_speed_bitps"
            };
            log.info("get {} cloudTowerHOST ", objects.size());
            List<Future<List<MetricFamilySamples>>> futures = new ArrayList<>();
            for (Object object : objects) {
                Future<List<MetricFamilySamples>> future = executorService.submit(() -> {
                    List<MetricFamilySamples> mfsOne = new ArrayList<>();
                    JSONObject server = JSONObject.parseObject(object.toString());
                    if (server.getString("uuid") == null) {
                        log.info("当前主机信息不完善{}" ,server.getString("name"));
                    } else {
                        GetHostNetworkMetricInput getHostNetworkMetricInput = new GetHostNetworkMetricInput()
                                .hosts(new HostWhereInput().id(server.getString("uuid")))
                                .range("2h")
                                .metrics(Arrays.asList(netMetrics))
                                .nics(new NicWhereInput().id(server.getString("nicUuid")));
                        List<WithTaskMetric> networkMetrics = metricsApi.getHostNetworkMetrics(getHostNetworkMetricInput);
                        for (WithTaskMetric monitor : networkMetrics) {
                            if (monitor.getData() == null){
                                continue;
                            }
                            Metric metric = monitor.getData();
                            if (metric.getSampleStreams().get(0).getPoints() == null) {
                                continue;
                            }
                            // 网口数据接收速度
                            if ("host_network_receive_speed_bitps".equals(metric.getSampleStreams().get(0).getLabels().getMetricName())) {
                                // 取时间最大值
                                double value = getV(metric);
                                networkReceive.addMetric(Arrays.asList(String.valueOf(cloudVendor.getId()), server.get("id").toString(),
                                        server.get("name").toString(), server.get("uuid").toString()), value);
                            }
                            // 网口数据发送速度
                            if ("host_network_transmit_speed_bitps".equals(metric.getSampleStreams().get(0).getLabels().getMetricName())) {
                                // 取时间最大值
                                double value = getV(metric);
                                networkTransmit.addMetric(Arrays.asList(String.valueOf(cloudVendor.getId()), server.get("id").toString(),
                                        server.get("name").toString(), server.get("uuid").toString()), value);
                            }
                        }
                        mfsOne.add(networkReceive);
                        mfsOne.add(networkTransmit);
                    }
                    return mfsOne;
                });
                futures.add(future);
            }
            Optional.ofNullable(futures).orElse(new ArrayList<>()).forEach(stringFuture -> {
                try {
                    mfs.addAll(stringFuture.get());
                } catch (Exception e) {
                    log.error("collect cloudTower hostNet partition metrics error!" + e.getMessage(), e);
                }
            });
            collectTimeUse.addMetric(Arrays.asList("cloudTower_host"), (System.currentTimeMillis() - now.getTime()));
            mfs.add(collectTimeUse);
        } catch (Exception e) {
            log.error(e.getMessage());
        }
        log.info("collect cloudTower hostNet partition metrics end! use " + (System.currentTimeMillis() - now.getTime()) / 1000 + "s");
    }

    private static Double getV(Metric metric) {
        return metric.getSampleStreams().get(metric.getSampleStreams().size() - 1)
                .getPoints()
                .get(metric.getSampleStreams().get(0).getPoints().size() - 1)
                .getV();
    }
}
