package com.bocloud.mcs.collector.cloudtower;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.ims.entity.CloudVendor;
import com.bocloud.mcs.common.Constants;
import com.google.common.util.concurrent.ThreadFactoryBuilder;
import com.smartx.tower.api.MetricsApi;
import com.smartx.tower.model.GetVmMetricInput;
import com.smartx.tower.model.Metric;
import com.smartx.tower.model.VmWhereInput;
import com.smartx.tower.model.WithTaskMetric;
import io.prometheus.client.GaugeMetricFamily;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;

import java.util.*;
import java.util.concurrent.*;

/**
 * archeros虚拟机监控采集器
 * @author byz
 * @version 1.0
 * @date 2023-06-30
 * @version 5.6.0-SNAPSHOT
 */
@Slf4j
@Component
public class CloudTowerVmCollector extends CloudTowerCollector {
    ThreadFactory nameThreadFactory = new ThreadFactoryBuilder().setNameFormat("cloudTower-vm-%d").build();
    private ExecutorService executorService = new ThreadPoolExecutor(Runtime.getRuntime().availableProcessors() + 1, Runtime.getRuntime().availableProcessors() + 1,
            0L, TimeUnit.MILLISECONDS,
            new LinkedBlockingQueue<>(), nameThreadFactory);
    @Override
    protected void collectMfs(List<MetricFamilySamples> mfs, MetricsApi metricsApi, CloudVendor cloudVendor) throws Exception {
        Date now = new Date();
        List<String> labels = Arrays.asList("vendorId", "id", "name", "instanceId");
        GaugeMetricFamily collectTimeUse = new GaugeMetricFamily(Constants.CLOUDTOWERVMPREFIX + "collect_time_use", "help",
                Arrays.asList("type"));
        try {
            BocloudResult bocloudResult = cmpCloudVendorService.vmtList(cloudVendor);
            JSONObject data = (JSONObject) bocloudResult.getData();
            if ( data == null) {
                log.info("查询cloudTower虚拟机为null");
                return;
            }
            JSONArray objects = JSONObject.parseArray(data.get("rows").toString());
            log.info("get {} cloudTowerVM ", objects.size());
            List<String> hostNames = new ArrayList<>();
            JSONObject jsonObject = new JSONObject();
            jsonObject.put("range","0");

            List<String> instanceIds = new ArrayList<>();
            List<Future<List<MetricFamilySamples>>> futures = new ArrayList<>();
            for (Object object : objects) {
                JSONObject server = JSONObject.parseObject(object.toString());
                if (server.get("instanceId") == null) {
                    log.info("当前云主机信息不完善{}" ,server.getString("name"));
                    continue;
                }
                instanceIds.add(server.getString("instanceId"));
            }

            String [] metrics = {
                    "elf_vm_cpu_overall_usage_percent",
                    "elf_vm_cpu_overall_used_hz",
                    "elf_vm_memory_usage_percent",
                    "elf_vm_memory_used_bytes",
                    "elf_vm_network_receive_packets",
                    "elf_vm_network_transmit_packets",
                    "elf_vm_disk_overall_read_iops",
                    "elf_vm_disk_overall_write_iops",
            };
            for (Object object : objects) {
                Future<List<MetricFamilySamples>> future = executorService.submit(() -> {
                    List<MetricFamilySamples> mfsOne = new ArrayList<>();
                    // cpu使用率
                    GaugeMetricFamily cpuUsed = new GaugeMetricFamily(Constants.CLOUDTOWERVMPREFIX + "elf_vm_cpu_overall_usage_percent", "help",
                            labels);
                    // 虚拟机CPU使用频率
                    GaugeMetricFamily cpuUsedTotal = new GaugeMetricFamily(Constants.CLOUDTOWERVMPREFIX + "elf_vm_cpu_overall_used_hz", "help",
                            labels);
                    // 内存使用率
                    GaugeMetricFamily memUsed = new GaugeMetricFamily(Constants.CLOUDTOWERVMPREFIX + "elf_vm_memory_usage_percent", "help",
                            labels);
                    // 虚拟机内存使用量
                    GaugeMetricFamily memUsedTotal = new GaugeMetricFamily(Constants.CLOUDTOWERVMPREFIX + "elf_vm_memory_used_bytes", "help",
                            labels);
                    // 网络吞吐量（流入）
                    GaugeMetricFamily netIngestionIn = new GaugeMetricFamily(Constants.CLOUDTOWERVMPREFIX + "elf_vm_network_receive_packets", "help",
                            labels);
                    // 网络吞吐量（流出）
                    GaugeMetricFamily netIngestionOut = new GaugeMetricFamily(Constants.CLOUDTOWERVMPREFIX + "elf_vm_network_transmit_packets", "help",
                            labels);
                    // 磁盘吞吐量（读）
                    GaugeMetricFamily diskIngestionRead = new GaugeMetricFamily(Constants.CLOUDTOWERVMPREFIX + "elf_vm_disk_overall_read_iops", "help",
                            labels);
                    // 磁盘吞吐量（写）
                    GaugeMetricFamily diskIngestionWrite = new GaugeMetricFamily(Constants.CLOUDTOWERVMPREFIX + "elf_vm_disk_overall_write_iops", "help",
                            labels);
                    JSONObject cloudServer = JSONObject.parseObject(object.toString());
                    if (cloudServer.get("instanceId") == null) {
                        log.info("当前云主机信息不完善{}" ,cloudServer.getString("name"));
                    } else {
                        GetVmMetricInput getVmMetricInput = new GetVmMetricInput();
                        getVmMetricInput.setMetrics(Arrays.asList(metrics));
                        getVmMetricInput.setRange("2h");
                        GetVmMetricInput monitors = getVmMetricInput.vms(new VmWhereInput().id(cloudServer.getString("instanceId")));
                        List<WithTaskMetric> vmMetrics = metricsApi.getVmMetrics(monitors);
                        for (WithTaskMetric monitor : vmMetrics) {
                            if (monitor.getData() == null){
                                continue;
                            }
                            Metric metric = monitor.getData();
                            if (metric.getSampleStreams().get(0).getPoints() == null) {
                                continue;
                            }
                            // cpu使用率
                            if ("elf_vm_cpu_overall_usage_percent".equals(metric.getSampleStreams().get(0).getLabels().getMetricName())) {
                                // 取时间最大值
                                double value = getV(metric);
                                cpuUsed.addMetric(Arrays.asList(String.valueOf(cloudVendor.getId()), cloudServer.get("id").toString(),
                                        cloudServer.get("name").toString(), cloudServer.get("instanceId").toString()), value);
                            }
                            if ("elf_vm_cpu_overall_used_hz".equals(metric.getSampleStreams().get(0).getLabels().getMetricName())) {
                                // 取时间最大值
                                double value = getV(metric);
                                cpuUsedTotal.addMetric(Arrays.asList(String.valueOf(cloudVendor.getId()), cloudServer.get("id").toString(),
                                        cloudServer.get("name").toString(), cloudServer.get("instanceId").toString()), value);
                            }
                            if ("elf_vm_memory_usage_percent".equals(metric.getSampleStreams().get(0).getLabels().getMetricName())) {
                                // 取时间最大值
                                double value = getV(metric);
                                memUsed.addMetric(Arrays.asList(String.valueOf(cloudVendor.getId()), cloudServer.get("id").toString(),
                                        cloudServer.get("name").toString(), cloudServer.get("instanceId").toString()), value);
                            }
                            if ("elf_vm_memory_used_bytes".equals(metric.getSampleStreams().get(0).getLabels().getMetricName())) {
                                // 取时间最大值
                                double value = getV(metric);
                                memUsedTotal.addMetric(Arrays.asList(String.valueOf(cloudVendor.getId()), cloudServer.get("id").toString(),
                                        cloudServer.get("name").toString(), cloudServer.get("instanceId").toString()), value);
                            }
                            if ("elf_vm_network_receive_packets".equals(metric.getSampleStreams().get(0).getLabels().getMetricName())) {
                                // 取时间最大值
                                double value = getV(metric);
                                netIngestionIn.addMetric(Arrays.asList(String.valueOf(cloudVendor.getId()), cloudServer.get("id").toString(),
                                        cloudServer.get("name").toString(), cloudServer.get("instanceId").toString()), value);
                            }
                            if ("elf_vm_network_transmit_packets".equals(metric.getSampleStreams().get(0).getLabels().getMetricName())) {
                                // 取时间最大值
                                double value = getV(metric);
                                netIngestionOut.addMetric(Arrays.asList(String.valueOf(cloudVendor.getId()), cloudServer.get("id").toString(),
                                        cloudServer.get("name").toString(), cloudServer.get("instanceId").toString()), value);
                            }
                            if ("elf_vm_disk_overall_read_iops".equals(metric.getSampleStreams().get(0).getLabels().getMetricName())) {
                                // 取时间最大值
                                double value = getV(metric);
                                diskIngestionRead.addMetric(Arrays.asList(String.valueOf(cloudVendor.getId()), cloudServer.get("id").toString(),
                                        cloudServer.get("name").toString(), cloudServer.get("instanceId").toString()), value);
                            }
                            if ("elf_vm_disk_overall_write_iops".equals(metric.getSampleStreams().get(0).getLabels().getMetricName())) {
                                // 取时间最大值
                                double value = getV(metric);
                                diskIngestionWrite.addMetric(Arrays.asList(String.valueOf(cloudVendor.getId()), cloudServer.get("id").toString(),
                                        cloudServer.get("name").toString(), cloudServer.get("instanceId").toString()), value);
                            }
                        }
                        mfsOne.add(cpuUsed);
                        mfsOne.add(cpuUsedTotal);
                        mfsOne.add(memUsed);
                        mfsOne.add(memUsedTotal);
                        mfsOne.add(diskIngestionRead);
                        mfsOne.add(diskIngestionWrite);
                        mfsOne.add(netIngestionIn);
                        mfsOne.add(netIngestionOut);
                    }
                    return mfsOne;
                });
                futures.add(future);
            }
            Optional.ofNullable(futures).orElse(new ArrayList<>()).forEach(stringFuture -> {
                try {
                    mfs.addAll(stringFuture.get());
                } catch (Exception e) {
                }
            });
            collectTimeUse.addMetric(Arrays.asList("cloudTower_vm"), (System.currentTimeMillis() - now.getTime()));
            mfs.add(collectTimeUse);
        } catch (Exception e) {
            log.error(e.getMessage());
        }
        log.info("collect cloudTower vm partition metrics end! use " + (System.currentTimeMillis() - now.getTime()) / 1000 + "s");
    }
    private static Double getV(Metric metric) {
        return metric.getSampleStreams().get(metric.getSampleStreams().size() - 1)
                .getPoints()
                .get(metric.getSampleStreams().get(0).getPoints().size() - 1)
                .getV();
    }
}

