package com.bocloud.mcs.collector.cloudtower;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.common.model.BocloudResult;
import com.bocloud.ims.entity.CloudVendor;
import com.bocloud.mcs.common.Constants;
import com.google.common.util.concurrent.ThreadFactoryBuilder;
import com.smartx.tower.api.MetricsApi;
import com.smartx.tower.model.*;
import io.prometheus.client.GaugeMetricFamily;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;

import java.util.*;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.*;

@Slf4j
@Component
public class CloudTowerVmNetCollector extends CloudTowerCollector {
    ThreadFactory nameThreadFactory = new ThreadFactoryBuilder().setNameFormat("cloudTower-vmNet-%d").build();
    private ExecutorService executorService = new ThreadPoolExecutor(Runtime.getRuntime().availableProcessors() + 1, Runtime.getRuntime().availableProcessors() + 1,
            0L, TimeUnit.MILLISECONDS,
            new LinkedBlockingQueue<>(), nameThreadFactory);
    @Override
    protected void collectMfs(List<MetricFamilySamples> mfs, MetricsApi metricsApi, CloudVendor cloudVendor) throws Exception {
        Date now = new Date();
        List<String> labels = Arrays.asList("vendorId", "id", "name", "instanceId");
        GaugeMetricFamily collectTimeUse = new GaugeMetricFamily(Constants.CLOUDTOWERVMPREFIX + "collect_time_use", "help",
                Arrays.asList("type"));
        try {
            BocloudResult bocloudResult = cmpCloudVendorService.vmtList(cloudVendor);
            JSONObject data = (JSONObject) bocloudResult.getData();
            if ( data == null) {
                log.info("查询cloudTower虚拟机为null");
                return;
            }
            JSONArray objects = JSONObject.parseArray(data.get("rows").toString());
            log.info("get {} cloudTowerVM ", objects.size());
            List<String> hostNames = new ArrayList<>();
            JSONObject jsonObject = new JSONObject();
            jsonObject.put("range","0");

            List<String> instanceIds = new ArrayList<>();
            String [] netMetrics = {
                    // 接收
                    "elf_vm_network_receive_speed_bps",
                    // 发送
                    "elf_vm_network_transmit_speed_bps"
            };
            List<Future<List<MetricFamilySamples>>> futures = new ArrayList<>();
            for (Object object : objects) {
                Future<List<MetricFamilySamples>> future = executorService.submit(() -> {
                    List<MetricFamilySamples> mfsOne = new ArrayList<>();
                    // 网口数据接收速度
                    GaugeMetricFamily vmNetReceived = new GaugeMetricFamily(Constants.CLOUDTOWERVMPREFIX + "elf_vm_network_receive_speed_bps", "help",
                            labels);
                    // 网口数据发送速度
                    GaugeMetricFamily vmNetTransmitted = new GaugeMetricFamily(Constants.CLOUDTOWERVMPREFIX + "elf_vm_network_transmit_speed_bps", "help",
                            labels);
                    JSONObject cloudServer = JSONObject.parseObject(object.toString());
                    if (cloudServer.get("instanceId") == null) {
                        log.info("当前云主机信息不完善{}" ,cloudServer.getString("name"));
                    } else {
                        GetVmNetWorkMetricInput getVmNetWorkMetricInput = new GetVmNetWorkMetricInput()
                                .range("2h")
                                .metrics(Arrays.asList(netMetrics))
                                .vms(new VmWhereInput().id(cloudServer.getString("instanceId")))
                                .nics(new VmNicWhereInput().id(Optional.ofNullable(cloudServer.getString("nicUuid")).orElse("")));
                        List<WithTaskMetric> vmNetMetrics = metricsApi.getVmNetWorkMetrics(getVmNetWorkMetricInput);
                        for (WithTaskMetric vmNetMetric : vmNetMetrics) {
                            if (vmNetMetric.getData() == null){
                                continue;
                            }
                            Metric metric = vmNetMetric.getData();
                            if (metric.getSampleStreams().get(0).getPoints() == null) {
                                continue;
                            }
                            // 网口数据接收速度
                            if ("elf_vm_network_receive_speed_bps".equals(metric.getSampleStreams().get(0).getLabels().getMetricName())) {
                                // 取时间最大值
                                double value = getV(metric);
                                vmNetReceived.addMetric(Arrays.asList(String.valueOf(cloudVendor.getId()), cloudServer.get("id").toString(),
                                        cloudServer.get("name").toString(), cloudServer.get("instanceId").toString()), value);
                            }
                            // 网口数据发送速度
                            if ("elf_vm_network_transmit_speed_bps".equals(metric.getSampleStreams().get(0).getLabels().getMetricName())) {
                                // 取时间最大值
                                double value = getV(metric);
                                vmNetTransmitted.addMetric(Arrays.asList(String.valueOf(cloudVendor.getId()), cloudServer.get("id").toString(),
                                        cloudServer.get("name").toString(), cloudServer.get("instanceId").toString()), value);
                            }
                        }
                        mfsOne.add(vmNetReceived);
                        mfsOne.add(vmNetTransmitted);
                    }
                    return mfsOne;
                });
                futures.add(future);
            }
            Optional.ofNullable(futures).orElse(new ArrayList<>()).forEach(stringFuture -> {
                try {
                    mfs.addAll(stringFuture.get());
                } catch (Exception e) {
                }
            });
            collectTimeUse.addMetric(Arrays.asList("cloudTower_vm"), (System.currentTimeMillis() - now.getTime()));
            mfs.add(collectTimeUse);
        } catch (Exception e) {
            log.error(e.getMessage());
        }
        log.info("collect cloudTower vmNet partition metrics end! use " + (System.currentTimeMillis() - now.getTime()) / 1000 + "s");

    }
    private static Double getV(Metric metric) {
        return metric.getSampleStreams().get(metric.getSampleStreams().size() - 1)
                .getPoints()
                .get(metric.getSampleStreams().get(0).getPoints().size() - 1)
                .getV();
    }
}
